#
# Must define MAIN in file that includes this (e.g., MAIN = l1.ltx)
# Can define GENFIGS to any extra figures that need to be generated
# and GENFIGSRC to any source
# Set PDF=pdf to use pdflatex

#
# Note:  some of the svg images assume that the computer modern tt
# 12-pt font is available.  Install it by copying cmtt12.pfb in
# directory ~/.fonts and running fc-cache.  (cmtt12.pfb is part of
# texlive, you can find it in a place like:
#
#   .../texmf-dist/fonts/type1/public/amsfonts/cm/cmtt12.pfb
#

#ifeq ($(origin PDF),undefined)
#PDF = pdf
#endif

ifeq ($(PDF),pdf)
FIGEXT	  = pdf
OUTEXT	  = pdf
else
FIGEXT	  = eps
OUTEXT	  = dvi
endif

PWD	  := $(shell pwd)
ifndef MAIN
MAIN	  := $(notdir $(PWD)).ltx
endif
BASE	  := $(basename $(MAIN))
SRCEXT	  := $(patsubst $(BASE).%,%,$(MAIN))
XFIGS	  := $(wildcard *.fig)
DIAFIGS	  := $(wildcard *.dia)
SVGFIGS	  := $(wildcard *.svg)
TEXFIGS	  := $(XFIGS:.fig=.tex) # $(DIAFIGS:.dia=.tex)
BINFIGS	  := $(XFIGS:.fig=.$(FIGEXT)) $(DIAFIGS:.dia=.$(FIGEXT)) \
		$(SVGFIGS:.svg=.$(FIGEXT)) $(GENFIGS)
STYLINK	  := $(notdir $(wildcard ../*.sty))
STYFILES  := $(sort $(STYLINK) $(wildcard *.sty))
TEXFILES  := $(sort $(MAIN) $(wildcard *.tex) $(TEXFIGS))
TEXDIST	  := $(filter-out $(TEXFIGS), $(TEXFIGS))
LINKFILES := $(STYLINK) $(EXTRALINK) genindex

JGRAPHS	  := $(wildcard *.j)
GGRAPHS	  := $(wildcard *.gnuplot)
GRAPHS	  := $(JGRAPHS.j=.$(FIGEXT)) $(GGRAPHS:.gnuplot=.$(FIGEXT))

ifeq ($(PDF),pdf)
IMAGESRC  := $(wildcard figs/*.gif) $(wildcard figs/*.eps) 
else
IMAGESRC  := $(wildcard figs/*.gif) $(wildcard figs/*.jpg)
endif
IMAGEGEN  := $(addsuffix .$(FIGEXT), $(basename $(IMAGESRC)))
IMAGES	  := $(sort $(IMAGEGEN) $(wildcard figs/*.$(FIGEXT)))
IMAGEDIST := $(filter-out $(IMAGEGEN), $(IMAGES))
# $(wildcard figs/*.odp) $(wildcard figs/*.ppt)

ifeq ($(PDF),pdf)
LATEX	  = pdflatex \\nonstopmode\\input
PS2PDF	  = ps2pdf
else
LATEX	  = latex \\nonstopmode\\input
#DVIPS	   = dvips -j0 -G0 -t letter
DVIPS	   = dvips -z -ta4
#DVIPS	  = dvips -z
#PS2PDF	   = GS_OPTIONS=-sPAPERSIZE=letter ps2pdf -sPAPERSIZE=letter
#PS2PDF	   = GS_OPTIONS="-sPAPERSIZE=a4" ps2pdf
PS2PDF	  = ps2pdf
endif

ifneq ($(origin NOADD),undefined)
ADD	  = : git add
else
ADD	  = git add
endif

all: $(BASE).ps $(BASE).pdf

ifeq ($(PDF),pdf)
%.pdf %.tex: %.fig
	fig2dev -L pdftex -p1 $< > $*.pdf
	fig2dev -L pdftex_t -p $*.pdf $< > $*.tex

%.pdf: %.eps
	perl -ne 'exit 1 unless /\n/' $< \
		|| perl -p -i -e 's/\r/\n/g' $<
	epstopdf --outfile=$@ $<

figs/%.pdf: figs/%.jpg
	convert $< $@

figs/%.pdf: figs/%.gif
	convert $< $@

#%.ps: %.pdf
#	pdf2ps $< $@

else
%.ps: %.dvi
	$(DVIPS) $< -o $@

%.eps %.tex: %.fig
	fig2dev -L pstex -p1 $< > $*.eps
	fig2dev -L pstex_t -p $*.eps $< > $*.tex
endif

%.eps: %.dia
	unset DISPLAY; dia -n -e $@ $<
%.tex: %.dia
	unset DISPLAY; dia -n -e $@ $<
%.eps: %.j
	@rm -f $@~
	jgraph $< > $@~
	mv -f $@~ $@
%.eps: %.gnuplot %.data
	gnuplot $*.gnuplot

%.eps: %.svg
	inkscape --export-eps=$@ $<

$(LINKFILES):
	rm -f $@
	ln -s ../$@ .

RERUN = egrep -q '(^LaTeX Warning:|\(natbib\)).* Rerun' $(basename $@).log
UNDEFINED = egrep -q '^(LaTeX|Package natbib) Warning:.* undefined' \
	$(basename $@).log

$(BASE).$(OUTEXT): \
%.$(OUTEXT): %.$(SRCEXT) $(TEXFILES) $(STYFILES) $(BINFIGS) $(GRAPHS) $(IMAGES)
	$(LATEX) $< || ! rm -f $@
	! $(UNDEFINED) || $(LATEX) $< || ! rm -f $@
	! $(RERUN) || $(LATEX) $< || ! rm -f $@
	! $(RERUN) || $(LATEX) $< || ! rm -f $@
ifeq ($(PDF),pdf)
	touch $*.dvi
	test ! -f .xpdf-running || xpdf -remote $(BASE)-server -reload
else

$(BASE).pdf: %.pdf: %.ps
	$(PS2PDF) $< $@
	test ! -f .xpdf-running || xpdf -remote $(BASE)-server -reload
endif

define PDF6UP
$(1)-print.tex: $(MAKEFILE_LIST) GNUmakefile $(EXTRAPRINT)
	rm -f $$@~
	echo '\batchmode' > $$@~
	echo '\documentclass[letterpaper]{article}' >> $$@~
	echo '\usepackage[utf8]{inputenc}\usepackage{pdfpages}' >> $$@~
	echo '\footskip 2.7cm' >> $$@~
	echo '\begin{document}' >> $$@~
	echo '\includepdf[pages=-,offset=0in 0in,nup=2x3,frame=false,noautoscale=false,delta=0.2cm 0.3cm,scale=0.95,pagecommand={\thispagestyle{empty}}]{$(1).pdf}' >> $$@~
	for file in $(EXTRAPRINT); do \
		echo "\includepdf[pages=-]{$$$$file}" >> $$@~; \
	done
	echo '\end{document}' >> $$@~
	mv -f $$@~ $$@

$(1)-print.pdf: $(1)-print.tex $(1).pdf
	pdflatex $$<

EXTRACLEAN += $(1)-print.*
EXTRAIGNORE += '$(1)-print.*'
endef

$(eval $(call PDF6UP,$(BASE)))
print: $(BASE)-print.pdf

DIST = GNUmakefile $(TEXFILES) $(XFIGS) $(DIAFIGS) $(SVGFIGS) $(GENFIGSRC) \
	$(JGRAPHS) $(GGRAPHS) $(GGRAPHS:.gnuplot=.data) \
	$(sort $(STYFILES) $(LINKFILES)) \
	$(IMAGESRC) $(IMAGEDIST) $(EXTRADIST)
dist: $(BASE).tar.gz
$(BASE).tar.gz: $(DIST) ../Lecture.mk
	tar -chzf $@ -C .. Lecture.mk \
		$(patsubst %, $(notdir $(PWD))/%, $(DIST)) \
	|| ! rm -f $@

preview: $(BASE).pdf
	if test -f .xpdf-running; then			\
		xpdf -remote $(BASE)-server -quit;	\
		! test -f .xpdf-running || sleep 0.1;	\
		! test -f .xpdf-running || sleep 1;	\
	fi
	touch .xpdf-running
	(xpdf -remote $(BASE)-server -z 100 $(BASE).pdf; rm -f .xpdf-running) &

#
# Make random-ish 6-hex-digit \extensioncode value by taking the first
# three bytes (6 hex characters) of the SHA1 hash of a bunch of system
# state.  (extensioncode must be manually added to the git repository)
#
extensioncode:
	(uptime; ps auxwww; netstat -a; netstat -s) \
		| (sha1 || sha1sum) 2> /dev/null \
		| cut -c 1-6 \
		| xargs printf '\\newcommand{\\theextensioncode}{%s}%%\n' > $@
$(BASE)-code.ltx:
	echo '\def\exposeextensioncode{}\input{$(BASE).ltx}' \
		> $(BASE)-code.ltx
$(BASE)-code.$(OUTEXT): $(BASE)-code.ltx $(BASE).$(OUTEXT)
	cp $(BASE).aux $(BASE)-code.aux
	$(LATEX) $(BASE)-code.ltx
ifneq ($(PDF),pdf)
$(BASE)-code.ps: $(BASE)-code.dvi
$(BASE)-code.pdf: $(BASE)-code.ps
	$(PS2PDF) $< $@
endif
ifeq ($(NEEDCODE), yes)
$(BASE).$(OUTEXT): extensioncode
$(eval $(call PDF6UP,$(BASE)-code))
print: $(BASE)-code-print.pdf
EXTRACLEAN += $(BASE)-code.ltx $(BASE)-code.pdf $(BASE)-code.ps
EXTRAIGNORE += $(BASE)-code.ltx $(BASE)-code.pdf $(BASE)-code.ps
endif

show: $(BASE).pdf
	xpdf -fullscreen $(BASE).pdf

osx: $(BASE).pdf
	open $(BASE).pdf

#EXTRAIGNORE = '*.aux' '*.bbl' '*.blg' '*.log' '*.dvi' '*.bak' '*.out'
ignore:
	rm -f .gitignore
	(set -f; for file in $(EXTRAIGNORE) \
			$(LINKFILES) .xpdf-running \
			$(BASE).tar.gz \
			$(TEXFIGS) $(BINFIGS) $(GRAPHS) $(IMAGEGEN) \
			$(BASE).ps $(BASE).pdf $(BASE)-print.*; do \
		echo "/$$file"; \
	done) | sort -u > .gitignore
	$(ADD) .gitignore

DESTDIR=../../www/notes
install: $(BASE).pdf $(BASE).tar.gz $(BASE)-print.pdf $(EXTRAINSTALL)
	test -d $(DESTDIR) || mkdir -p $(DESTDIR)
	topic=`sed -ne 's/^Lecture Topic: *//p' $(BASE).log` \
		&& test -n "$$topic" \
		&& ./genindex $(DESTDIR)/index.html $(BASE) "$$topic" \
			$(EXTRAINSTALL)
	cp $^ $(DESTDIR)/
	$(ADD) $(patsubst %, $(DESTDIR)/%, $^) $(DESTDIR)/index.html

clean:
	rm -f $(BASE).ps $(BASE).pdf $(BASE)-print.* $(BASE).tar.gz
	rm -f $(TEXFIGS) $(BINFIGS) $(GRAPHS) $(IMAGEGEN) $(EXTRACLEAN)
	for file in $(LINKFILES); do \
		! test -h $$file -a -f ../$$file || rm -f $$file; \
	done
	rm -f *.dvi *.aux *.log *.bbl *.blg *.lof *.lot *.toc *.bak *.out
	rm -f *~ *.core core

always:
	@:

.PHONY: install clean all always ignore preview show osx dist print
